@@ -1,6 +1,6 @@
 /*
  * ProFTPD - mod_proxy SSH message format
- * Copyright (c) 2021-2022 TJ Saunders
+ * Copyright (c) 2021-2024 TJ Saunders
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,7 +70,7 @@ uint32_t proxy_ssh_msg_read_byte(pool *p, unsigned char **buf, uint32_t *buflen,
 }
 
 uint32_t proxy_ssh_msg_read_bool(pool *p, unsigned char **buf, uint32_t *buflen,
-    int *bool) {
+    int *b) {
   unsigned char byte = 0;
   uint32_t len;
 
@@ -81,7 +81,7 @@ uint32_t proxy_ssh_msg_read_bool(pool *p, unsigned char **buf, uint32_t *buflen,
     return 0;
   }
 
-  *bool = byte;
+  *b = byte;
   return len;
 }
 
@@ -355,8 +355,8 @@ uint32_t proxy_ssh_msg_write_byte(unsigned char **buf, uint32_t *buflen,
 }
 
 uint32_t proxy_ssh_msg_write_bool(unsigned char **buf, uint32_t *buflen,
-    unsigned char bool) {
-  return proxy_ssh_msg_write_byte(buf, buflen, bool == 0 ? 0 : 1);
+    unsigned char b) {
+  return proxy_ssh_msg_write_byte(buf, buflen, b == 0 ? 0 : 1);
 }
 
 uint32_t proxy_ssh_msg_write_data(unsigned char **buf, uint32_t *buflen,
